<?php
namespace Teknisa\PED\Modules\Pedidos\Controllers;

    use \Teknisa\GER\Libraries\PHPMailer;
    use Teknisa\PED\Controllers\TeknisaCrud;
    use Teknisa\GER\Controllers\CrudController;
    use Teknisa\PED\Modules\Pedidos\Services\ped02300PrecoBaseService;
    use Zeedhi\Framework\DTO;
    use Zeedhi\Framework\DTO\Response\Error;
    use \Zeedhi\Framework\DataSource\DataSet;
    use Zeedhi\Framework\DataSource\ParameterBag;

    class ped02300PrecoBaseController extends TeknisaCrud {
        protected $dataSourceName = 'preco_base_produto';
        protected $dataSourceNameToSave = '';
        protected $newCode;
        protected $environment;
        protected $PrecoBaseService;

        public function __construct($crudService, ParameterBag $parameterBag, $newCode, $environment, $PrecoBaseService){
            $this->newCode = $newCode;
            $this->environment = $environment;
            $this->parameterBag = $parameterBag;
            $this->PrecoBaseService = $PrecoBaseService;
            parent::__construct($crudService, $parameterBag);
        }

        public function saveItemPreBase(DTO\Request\DataSet $request, DTO\Response $response){
            try {
                $this->dataSourceName = 'itemprebase';
                parent::save($request, $response);
            } catch (\Exception $e) {
                $response->setError(DTO\Response\Error($e->getMessage()));
            }
        }

        public function deleteItemPreBase($request, DTO\Response $response){
            try {
                $this->dataSourceName = 'itemprebase';
                parent::delete($request,$response);
            } catch (\Exception $e) {
                $response->setError(DTO\Response\Error($e->getMessage()));
            }
        }

        public function findPromoPreBase($request, $response){
            try {
                $this->dataSourceName = 'promo_prebase';
                parent::find($request,$response);
            } catch (\Exception $e) {
                $response->setError(DTO\Response\Error($e->getMessage()));
            }
        }

        public function deletePromoPreBase($request, DTO\Response $response){
            try {
                $this->dataSourceName = 'promoprbase';
                parent::delete($request,$response);
            } catch (\Exception $e) {
                $response->setError(DTO\Response\Error($e->getMessage()));
            }
        }

        public function findPrecoOpcional($request, $response){
            try {
                $this->dataSourceName = 'preco_opcional';
                parent::find($request,$response);
            } catch (\Exception $e) {
                $response->setError(DTO\Response\Error($e->getMessage()));
            }
        }

        public function savePrecoOpcional(DTO\Request\DataSet $request, DTO\Response $response){
            try {
                $this->dataSourceName = 'opcioprbase';
                parent::save($request, $response);
            } catch (\Exception $e) {
                $response->setError(DTO\Response\Error($e->getMessage()));
            }
        }

        public function deletePrecoOpcional($request, DTO\Response $response){
            try {
                $this->dataSourceName = 'opcioprbase';
                parent::delete($request,$response);
            } catch (\Exception $e) {
                $response->setError(DTO\Response\Error($e->getMessage()));
            }
        }

        public function findFatorAjusteProduto($request, $response){
            try {
                $this->dataSourceName = 'fator_ajuste_produto';
                parent::find($request,$response);
            } catch (\Exception $e) {
                $response->setError(DTO\Response\Error($e->getMessage()));
            }
        }

        public function saveFatorAjusteProduto(DTO\Request\DataSet $request, DTO\Response $response){
            try {
                $this->dataSourceName = 'prodfator';
                parent::save($request, $response);
            } catch (\Exception $e) {
                $response->setError(DTO\Response\Error($e->getMessage()));
            }
        }

        public function deleteFatorAjusteProduto($request, DTO\Response $response){
            try {
                $this->dataSourceName = 'prodfator';
                parent::delete($request,$response);
            } catch (\Exception $e) {
                $response->setError(DTO\Response\Error($e->getMessage()));
            }
        }

        public function findPrBaseConvidaore($request, $response){
            try {
                $this->dataSourceName = 'prbase_convidaore';
                parent::find($request,$response);
            } catch (\Exception $e) {
                $response->setError(DTO\Response\Error($e->getMessage()));
            }
        }

        public function savePrBaseConvidaore(DTO\Request\DataSet $request, DTO\Response $response){
            try {
                $this->dataSourceName = 'prbaseconvidaore';
                parent::save($request, $response);
            } catch (\Exception $e) {
                $response->setError(DTO\Response\Error($e->getMessage()));
            }
        }

        public function deletePrBaseConvidaore($request, DTO\Response $response){
            try {
                $this->dataSourceName = 'prbaseconvidaore';
                parent::delete($request,$response);
            } catch (\Exception $e) {
                $response->setError(DTO\Response\Error($e->getMessage()));
            }
        }

        public function validaIntervalo($request, DTO\Response $response){
            $dataSet = $request->getDataSet();
            $rows = $dataSet->getRows();
            $nrorg = $this->environment->getNrOrgTrab();
            $valida = $this->PrecoBaseService->validaIntervalo($rows,$nrorg);

            try {
                if($valida != null){
                    throw new \Exception("Intervalo de convidados está contido parcial ou totalmente em outro já cadastrado.");
                } else {
                    return true;
                }
            } catch (\Exception $e) {
            // throw $e;
            // substituir throw por setErro para corrigir mensagem que incluia a string Exception
               $response->setError(new Error($e->getMessage(), 500));
            }
        }

        public function exportarItens($request, DTO\Response $response){
            $dataSet = $request->getDataSet();
            $rows = $dataSet->getRows();
            try {
                $this->PrecoBaseService->insereIten($rows);
            } catch (\Exception $e) {
                if(!(count($response->getError()))) {
                    $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
                }
            }
        }

        public function reajusteItem($request, DTO\Response $response){
            $dataSet = $request->getDataSet();
            $rows = $dataSet->getRows();
            try {
                $this->PrecoBaseService->reajusteIten($rows);
            } catch (\Exception $e) {
            }
        }
        
        public function saveFatorPreco($request, DTO\Response $response){
            try {
            $row = $request->getRow();
            $novoCodigo = $this->newCode->getCodeByTableName("FATORPRECO", $this->environment->getNrOrgTrab());
            
            $row['CDFATPRECO'] = substr($novoCodigo, -3);
            
            $this->dataSourceName = 'fatorpreco';
            parent::save($request, $response);
                
            } catch (\Exception $e) {
                throw new \Exception('Erro ao salvar fator de preço.');
            }
        }
        
        public function newCode($request, DTO\Response $response){
            try{
                $novoCodigo = $this->newCode->getCodeByTableName("PROMOPRBASE", $this->environment->getNrOrgTrab());
                $newCode = array();
                array_push($newCode, $novoCodigo);
                //var_dump($newCode);
                $dataSet = new DataSet ('data',$newCode);
                $response->addDataSet($dataSet);
                
            }catch (\Exception $e){
                throw $e;
            }
        }
        
        //-----------------------------------------------------------------------------------------------------//
        protected function processConditionValue($condition)
        {
            return $condition['value'];
        }

        protected function populateParameterBag(FilterCriteria $filterCriteria)
        {

            $conditions = array();
            foreach ($filterCriteria->getConditions() as $condition) {
                $columnName = $condition['columnName'];
                if (in_array($columnName, $this->parameterBagColumns)) {
                    $this->parameterBag->set($columnName, $this->processConditionValue($condition));
                } else {
                    $conditions[] = $condition;
                }
            }
            $newFilterCriteria = new FilterCriteria(
                $filterCriteria->getDataSourceName(),
                $conditions,
                $filterCriteria->getPage(),
                $filterCriteria->getPageSize()
            );
            return $newFilterCriteria;
        }
        
        public function updateDescMaximo($request, DTO\Response $response){
            $dataSet = $request->getDataSet();
            $rows = $dataSet->getRows();
            $cdprecobase = $rows[0]['CDPRECOBASE'] ;
            $sgestadoAntigo = $rows[0]['SGESTADO'] ;
            $cdprodutoAntigo = $rows[0]['CDPRODUTO'] ;

            try {
                $this->PrecoBaseService->updateDescMaximo($rows[1], $cdprecobase, $sgestadoAntigo, $cdprodutoAntigo);
            } catch (\Exception $e) {
               throw $e;
            }
        }
        
        public function salvarCadAutoFator($request, DTO\Response $response){
            $row = $request->getRow();
            
            $this->PrecoBaseService->salvarCadAutoFator($row, $response);
            
        }
    }
?>