<?php
namespace Teknisa\PED\Modules\Pedidos\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\ORM\DateTime;
use \Teknisa\Libs\Util\DateUtil;


class Ped05700importacaoSolicitacaoComprasFactory {
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection       = $entityManager->getConnection();
        $this->entityManager    = $entityManager;
    }
     
    public function buscaPrecoBase($cdfilial, $cdcliente, $cdvendedor, $cdmodpgto){
        try{
            $sql = 'SELECT BUSCA_PRECOBASE(:P_CDFILIAL, :P_CDCLIENTE, :P_CDVENDEDOR, :P_CDMODPGTO) AS CDPRECOBASE FROM DUAL';
            
            $params = array(
                'P_CDFILIAL' => $cdfilial, 
                'P_CDCLIENTE' => $cdcliente, 
                'P_CDVENDEDOR' => $cdvendedor, 
                'P_CDMODPGTO' => $cdmodpgto
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function buscaitemSoliRelacProdSC($cdfilsoli, $cdfillan, $nrsolicmp, $dtprogentr, $tdtprogentr, $cdfilial){
        try{
            $sql = "SELECT I.CDPRODUTO,I.DTPROGENTR, P.CDARVPROD, P.NMPRODUTO
                          FROM ITEMSOLI I, PRODUTO P
                         WHERE CDFILSOLI = :P_CDFILSOLI
                           AND CDFILLAN  = :P_CDFILLAN
                           AND NRSOLICMP = :P_NRSOLICMP
                           AND ((I.DTPROGENTR = :P_DTPROGENTR) OR (:P_TDTPROGENTR = 'T'))
                           AND I.CDPRODUTO = P.CDPRODUTO
                           AND PROD_RELAC_SC(:P_CDFILIAL, I.CDPRODUTO) IS NULL";
                       
            $params = array(
                'P_CDFILSOLI' => $cdfilsoli,
                'P_CDFILLAN' => $cdfillan,
                'P_NRSOLICMP' => $nrsolicmp,
                'P_DTPROGENTR' => $dtprogentr,
                'P_TDTPROGENTR' => 'T',
                'P_CDFILIAL' => $cdfilial
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }

    public function buscaProdFatorDtEnt($cdfilsoli, $cdfillan, $nrsolicmp, $dtprogentr, $cdfilial,$cdfatpreco){
        try{
            $sql = "SELECT I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO
                    FROM ITEMSOLI I, PRODUTO P
                    WHERE I.CDFILSOLI = :P_CDFILSOLI
                    AND I.CDFILLAN  = :P_CDFILLAN
                    AND I.NRSOLICMP = :P_NRSOLICMP
                    AND I.DTPROGENTR = :P_DTPROGENTR
                    AND PROD_RELAC_SC(:P_CDFILIAL, I.CDPRODUTO) = P.CDPRODUTO
                    AND BUSCA_FATOR_PRECO(:P_CDFATPRECO, PROD_RELAC_SC(:P_CDFILIAL, I.CDPRODUTO)) <= 0";
            
            $params = array(
                'P_CDFILIAL' => $cdfilial, 
                'P_NRSOLICMP' => $nrsolicmp,
                'P_CDFILLAN' => $cdfillan,
                'P_DTPROGENTR' => $dtprogentr, 
                'P_CDFILSOLI' => $cdfilsoli,
                'P_CDFATPRECO' => $cdfatpreco
            );
            
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function buscaPrecoBaseDtEnt($cdfilial, $nrsolicmp, $cdfillan, $dtprogentr, $cdfilsoli, $precoBase, $cdcliente, $cdmodpgto){
        try{
            $sql = "SELECT I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO
                      FROM ITEMSOLI I, PRODUTO P
                     WHERE CDFILSOLI = :P_CDFILSOLI
                       AND CDFILLAN  = :P_CDFILLAN
                       AND NRSOLICMP = :P_NRSOLICMP
                       AND DTPROGENTR = :P_DTPROGENTR
                       AND PROD_RELAC_SC(:P_CDFILIAL, I.CDPRODUTO) = P.CDPRODUTO
                       AND BUSCA_PRECO_PED(:P_CDFILIAL, P.CDARVPROD, :P_CDCLIENTE, :P_CDPRECOBASE, I.DTPROGENTR, :P_CDMODPGTO) <= 0";
            
            $params = array(
                'P_CDFILIAL' => $cdfilial, 
                'P_NRSOLICMP' => $nrsolicmp,
                'P_CDFILLAN' => $cdfillan,
                'P_DTPROGENTR' => substr($dtprogentr, 0, 10), 
                'P_CDFILSOLI' => $cdfilsoli,
                'P_CDCLIENTE' => $cdcliente,
                'P_CDPRECOBASE' => $precoBase,
                'P_CDMODPGTO' => $cdmodpgto
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function buscaItemSoli($nrsolicmp, $tdtprogentr, $cdgrupo, $cdsubgrprod, $tcdsubgrprod,  $cdfillan, $dtprogentr, $cdfilsoli){
        try{
            $sql = "SELECT DISTINCT
                           I.CDPRODUTO, I.DTPROGENTR, P.NMPRODUTO,
                           NVL(QTSOLIPROD,0) QTSOLIPROD
                      FROM ITEMSOLI I, PRSUBGRPROD PS, PRODUTO P
                     WHERE CDFILSOLI = :P_CDFILSOLI
                       AND CDFILLAN  = :P_CDFILLAN
                       AND NRSOLICMP = :P_NRSOLICMP
                       AND(P.CDPRODUTO = I.CDPRODUTO)
                       AND (PS.CDPRODUTO(+) = I.CDPRODUTO)
                       AND ((PS.CDGRUPPROD = :P_CDGRUPO ) OR (:P_CDGRUPO IS NULL))
                       AND ((INSTR(:P_CDSUBGRPROD, CDSUBGRPROD) > 0) OR (:P_TCDSUBGRPROD ='T'))
                       AND ((I.DTPROGENTR = :P_DTPROGENTR) OR (:P_TDTPROGENTR = 'T'))
                     ORDER BY DTPROGENTR";

            $params = array(
                'P_CDFILSOLI' => $cdfilsoli,
                'P_CDFILLAN' => $cdfillan,
                'P_NRSOLICMP' => $nrsolicmp,
                'P_CDGRUPO' => $cdgrupo, 
                'P_CDSUBGRPROD'=> $cdsubgrprod,
                'P_TCDSUBGRPROD' => $tcdsubgrprod,
                'P_DTPROGENTR' => $dtprogentr, 
                'P_TDTPROGENTR' => $tdtprogentr
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function buscaFator($sgfatpreco, $tsgfatpreco){
        try{
            $sql = "SELECT
                CDFATPRECO, SGFATPRECO, NMFATPRECO
                FROM FATORPRECO
                WHERE (SGFATPRECO = :P_SGFATPRECO OR :P_TSGFATPRECO = 'T')
                ORDER BY SGFATPRECO";
                
            $params = array(
                'P_SGFATPRECO' => $sgfatpreco,
                'P_TSGFATPRECO' => $tsgfatpreco
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function buscaProdFator($cdfilial, $cdfilsoli, $cdfillan, $nrsolicmp, $fator){
        try{
            $sql = "SELECT
                    P.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO
                    FROM PRODUTO P,
                    (SELECT DISTINCT
                            PROD_RELAC_SC(:P_CDFILIAL, CDPRODUTO) AS CDPRODUTO
                    FROM ITEMSOLI
                   WHERE CDFILSOLI = :P_CDFILSOLI
                     AND CDFILLAN  = :P_CDFILLAN
                     AND NRSOLICMP = :P_NRSOLICMP
                    MINUS
                    SELECT
                    CDPRODUTO
                    FROM PRODFATOR
                   WHERE CDFATPRECO = :P_CDFATPRECO)X
                   WHERE X.CDPRODUTO = P.CDPRODUTO
                   ORDER BY CDPRODUTO";
    
            $params = array(
                'P_CDFILIAL' => $cdfilial,
                'P_CDFILSOLI' => $cdfilsoli,
                'P_CDFILLAN' => $cdfillan,
                'P_NRSOLICMP' => $nrsolicmp,
                'P_CDFATPRECO' => $fator
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function selectPrecoBase($cdfilial, $cdfilsoli, $cdfillan, $nrsolicmp, $cdprecobase){
        try{
                $sql = "SELECT
                           P.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO
                        FROM
                           PRODUTO P ,
                           (
                              SELECT DISTINCT PROD_RELAC_SC(:P_CDFILIAL, CDPRODUTO) AS CDPRODUTO
                              FROM ITEMSOLI
                              WHERE
                                 CDFILSOLI = :P_CDFILSOLI
                                 AND CDFILLAN  = :P_CDFILLAN
                                 AND NRSOLICMP = :P_NRSOLICMP
                              MINUS
                              SELECT
                                 CDPRODUTO
                              FROM
                                 ITEMPREBASE
                              WHERE
                                 CDPRECOBASE = :P_CDPRECOBASE)X
                        WHERE
                           X.CDPRODUTO = P.CDPRODUTO
                        ORDER BY CDPRODUTO";
    
            $params = array(
                'P_CDFILIAL' => $cdfilial,
                'P_CDFILSOLI' => $cdfilsoli,
                'P_CDFILLAN' => $cdfillan,
                'P_NRSOLICMP' => $nrsolicmp,
                'P_CDPRECOBASE' => $cdprecobase
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function selectDataEndPed($cdfilsoli, $cdfillan, $nrsolicmp){
        try{
            $sql = "SELECT 
                        MIN(DTPROGENTR) AS DTENTREGA
                    FROM 
                       ITEMSOLI
                    WHERE 
                        CDFILSOLI = :P_CDFILISOLI
                    AND CDFILLAN  = :P_CDFILLAN 
                    AND NRSOLICMP = :P_NRSOLICMP";
    
            $params = array(
                'P_CDFILISOLI' => $cdfilsoli,
                'P_CDFILLAN' => $cdfillan,
                'P_NRSOLICMP' => $nrsolicmp
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function selectValidaPreco($cdfilial, $cdfilsoli, $cdfillan, $nrsolicmp, $stResult, $dtentrega){
        try{
            $sql = "SELECT
                   P.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO
                FROM
                   PRODUTO P ,
                   (
                      SELECT DISTINCT PROD_RELAC_SC(:P_CDFILIAL, CDPRODUTO) AS CDPRODUTO
                      FROM ITEMSOLI
                      WHERE
                         CDFILSOLI = :P_CDFILSOLI
                         AND CDFILLAN  = :P_CDFILLAN
                         AND NRSOLICMP = :P_NRSOLICMP
                      MINUS
                      SELECT
                         CDPRODUTO
                      FROM
                         ITEMPREBASE
                      WHERE
                         CDPRECOBASE = :P_CDPRECOBASE
                     AND (DTINIVALIPB <= :P_DTPEDIDO
                     AND :P_DTPEDIDO <= NVL(DTFIMVALIPB, TO_DATE('31/12/9999')))
                )X
                WHERE
                   X.CDPRODUTO = P.CDPRODUTO
                ORDER BY
                   CDPRODUTO";
    
            $params = array(
                'P_CDFILIAL' => $cdfilial,
                'P_CDFILSOLI' => $cdfilsoli,
                'P_CDFILLAN' => $cdfillan,
                'P_NRSOLICMP' => $nrsolicmp,
                'P_CDPRECOBASE' => $stResult, 
                'P_DTPEDIDO' => $dtentrega
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function selectValidaPrecoZerado($cdfilial, $cdfilsoli, $cdfillan, $nrsolicmp, $cdprecobase, $dtentrega){
        try{
            $sql = "SELECT
                   P.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO
                FROM
                   PRODUTO P ,
                   (
                      SELECT DISTINCT PROD_RELAC_SC(:P_CDFILIAL, CDPRODUTO) AS CDPRODUTO
                      FROM ITEMSOLI
                      WHERE
                         CDFILSOLI = :P_CDFILSOLI
                         AND CDFILLAN  = :P_CDFILLAN
                         AND NRSOLICMP = :P_NRSOLICMP
                      MINUS
                      SELECT
                         CDPRODUTO
                      FROM
                         ITEMPREBASE
                      WHERE
                         CDPRECOBASE = :P_CDPRECOBASE
                     AND (DTINIVALIPB <= :P_DTPEDIDO
                     AND :P_DTPEDIDO <= NVL(DTFIMVALIPB, TO_DATE('31/12/9999')))
                     AND VRITEPREBASE > 0
                )X
                WHERE
                   X.CDPRODUTO = P.CDPRODUTO
                ORDER BY
                   CDPRODUTO";
    
            $params = array(
                'P_CDFILIAL' => $cdfilial,
                'P_CDFILSOLI' => $cdfilsoli,
                'P_CDFILLAN' => $cdfillan,
                'P_NRSOLICMP' => $nrsolicmp,
                'P_CDPRECOBASE' => $cdprecobase, 
                'P_DTPEDIDO' => $dtentrega
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function selectCalcTotImpos($cdfilial, $nrpedido){
        try{
            $sql = "SELECT CDFILIAL,
                           NRPEDIDO,
                           CDIMPOSTO,
                           SUM(VRIMPOSTOITPED) AS VRIMPOSTOTAL,
                           SUM(VRBASECALCITPED) AS VRBASETOTAL
                      FROM IMPITEMPEDIDO
                     WHERE CDFILIAL  = :P_CDFILIAL AND
                           NRPEDIDO  = :P_NRPEDIDO
                     GROUP BY CDFILIAL, NRPEDIDO, CDIMPOSTO";
            
            $params = array(
                'P_CDFILIAL' => $cdfilial, 
                'P_NRPEDIDO' => $nrpedido
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }

    public function selectConsImposPedTot($cdfilial, $nrpedido, $cdimposto){
        try{
            $sql = "SELECT I.CDFILIAL,
                           I.NRPEDIDO,
                           I.CDIMPOSTO,
                           I.VRTOTIMPPED,
                           I.VRTOTBCALCPED,
                           T.SGIMPOSTO,
                           T.IDTRATIMPO
                      FROM IMPOSNFTOTPED I, IMPOSTO T
                     WHERE I.CDFILIAL  = :P_CDFILIAL  AND
                           I.NRPEDIDO  = :P_NRPEDIDO  AND
                           I.CDIMPOSTO = :P_CDIMPOSTO AND
                           I.CDIMPOSTO = T.CDIMPOSTO";
            
            $params = array(
                'P_CDFILIAL' => $cdfilial, 
                'P_NRPEDIDO' => $nrpedido,
                'P_CDIMPOSTO'=> $cdimposto
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }

    public function insertItemPedTot($cdfilial, $nrpedido, $cdimposto, $vrtotimpped, $vrtotbcalcped){
        try{
            $sql = "INSERT INTO IMPOSNFTOTPED
                                (CDFILIAL, NRPEDIDO, CDIMPOSTO,
                                 VRTOTIMPPED, VRTOTBCALCPED)
                                VALUES
                                (:CDFILIAL, :NRPEDIDO, :CDIMPOSTO,
                                 :VRTOTIMPPED, :VRTOTBCALCPED)";
            
            $params = array(
                'CDFILIAL' => $cdfilial, 
                'NRPEDIDO' => $nrpedido,
                'CDIMPOSTO'=> $cdimposto,
                'VRTOTIMPPED'=> $vrtotimpped,
                'VRTOTBCALCPED' => $vrtotbcalcped
            );
            
            $std = $this->connection->prepare($sql);
            return $std->execute($params);
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }

    public function updateItemPedTot($cdfilial, $nrpedido, $cdimposto, $vrtotimpped, $vrtotbcalcped){
        try{
            $sql = "UPDATE IMPOSNFTOTPED
                           SET VRTOTIMPPED  = :VRTOTIMPPED,
                               VRTOTBCALCPED = :VRTOTBCALCPED
                         WHERE CDFILIAL  = :CDFILIAL AND
                               NRPEDIDO  = :NRPEDIDO AND
                               CDIMPOSTO = :CDIMPOSTO ";
            
            $params = array(
                'CDFILIAL' => $cdfilial, 
                'NRPEDIDO' => $nrpedido,
                'CDIMPOSTO'=> $cdimposto,
                'VRTOTIMPPED'=> $vrtotimpped,
                'VRTOTBCALCPED' => $vrtotbcalcped
            );
            
            $std = $this->connection->prepare($sql);
            return $std->execute($params);
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }

    public function selectExisteSoliCmpDtPed($cdfillanca, $cdfilisoli, $nrsolicmp, $dtprogentr){
        try{
            $sql = "SELECT COUNT(1) AS QTRETORNO
                      FROM PEDIDO
                     WHERE CDFILLANCA = :P_CDFILLANCA
                       AND CDFILSOLI  = :P_CDFILSOLI
                       AND NRSOLICMP  = :P_NRSOLICMP
                       AND DTENTPED = :P_DTPROGENTR";
            
            $params = array(
                'P_CDFILLANCA' => $cdfillanca, 
                'P_CDFILSOLI' => $cdfilisoli,
                'P_NRSOLICMP'=> $nrsolicmp,
                'P_DTPROGENTR'=> $dtprogentr,
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function selectExisteSoliCmpPed($cdfillanca, $cdfilisoli, $nrsolicmp, $dtprogentr){
        try{
            $sql = "SELECT COUNT(1) AS QTRETORNO
                  FROM PEDIDO
                 WHERE CDFILLANCA = :P_CDFILLANCA
                   AND CDFILSOLI  = :P_CDFILSOLI
                   AND NRSOLICMP  = :P_NRSOLICMP
                   AND DTENTPED = :P_DTPROGENTR";
            
            $params = array(
                'P_CDFILLANCA' => $cdfillanca, 
                'P_CDFILSOLI' => $cdfilisoli,
                'P_NRSOLICMP'=> $nrsolicmp,
                'P_DTPROGENTR'=> $dtprogentr
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function gravaItempedido($cdfilial, $nrpedido, $cdfatpreco, $cdproduto, $qtitemped, $dtentitped, $dtentped){
        try {

            $this->entityManager->getConnection()->beginTransaction();
            //$dtentitped = DateUtil::getDataDeString($dtentitped , "d/m/Y H:i:s")->format('d/m/Y');
            //$dtentped = DateUtil::getDataDeString($dtentped, "d/m/Y H:i:s")->format('d/m/Y');
            $dtentitped =  substr($dtentitped, 0, 10);
            $dtentped =  substr($dtentped, 0, 10);

            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'GRAVA_ITEMPEDIDO');
            
            $procedure->addParam(new Param('P_CDFILIAL'        , Param::PARAM_INPUT, $cdfilial       , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPEDIDO'        , Param::PARAM_INPUT, $nrpedido       , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTO'       , Param::PARAM_INPUT, $cdproduto      , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_QTITEMPED'       , Param::PARAM_INPUT, $qtitemped      , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRPREUNITPED'    , Param::PARAM_INPUT, null            , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFATPRECO'      , Param::PARAM_INPUT, $cdfatpreco     , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRDESITPED'      , Param::PARAM_INPUT, null            , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRPESOITPED'     , Param::PARAM_INPUT, null            , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRPRTABITPED'    , Param::PARAM_INPUT, null            , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRPEDEMAXIPE'    , Param::PARAM_INPUT, null            , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTENTITPED'      , Param::PARAM_INPUT, $dtentitped     , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTENTPED'        , Param::PARAM_INPUT, $dtentped       , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRITEMORCAMENTO' , Param::PARAM_INPUT, null            , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMENSERRO'      ,Param::PARAM_OUTPUT, ''              , Param::PARAM_TYPE_STR, 1000));
            $procedure->addParam(new Param('P_NRITEMPED'       ,Param::PARAM_OUTPUT, ''              , Param::PARAM_TYPE_STR, 1000));
            

            $retorno = $procedure->execute(); 
            $this->entityManager->getConnection()->commit();
            return $retorno; 

        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function gravaPedido($cdfilial, $prxPedido, $operador, $dtprogentr, $cdfillanca, $cdfilisoli, $nrsolicmp) {
        try {
            $this->entityManager->getConnection()->beginTransaction();
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'GRAVA_PEDIDO');

            $procedure->addParam(new Param('P_CDFILIAL'      , Param::PARAM_INPUT, $cdfilial    , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPEDIDO'      , Param::PARAM_INPUT, $prxPedido   , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDCLIENTE'     , Param::PARAM_INPUT, null         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMODPGTO'     , Param::PARAM_INPUT, null         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTPEDIDO'      , Param::PARAM_INPUT, null         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTENTPED'      , Param::PARAM_INPUT, $dtprogentr , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRINSJURTRAN'  , Param::PARAM_INPUT, null         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDTPIJURTRAN'  , Param::PARAM_INPUT, null         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPELANPED'   , Param::PARAM_INPUT, $operador    , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIALPED'   , Param::PARAM_INPUT, $cdfilial    , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDTIPORECE'    , Param::PARAM_INPUT, null         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDVENDEDOR'    , Param::PARAM_INPUT, null         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDTIPOOPER'    , Param::PARAM_INPUT, null         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRECOBASE'   , Param::PARAM_INPUT, null         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILLANCA'    , Param::PARAM_INPUT, $cdfillanca  , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILSOLI'     , Param::PARAM_INPUT, $cdfilisoli  , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSOLICMP'     , Param::PARAM_INPUT, $nrsolicmp   , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDTIPOEDITAL'  , Param::PARAM_INPUT, null         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NREMPENHO'     , Param::PARAM_INPUT, null         , Param::PARAM_TYPE_STR)); 
            $procedure->addParam(new Param('P_CDFILIORCEVE'  , Param::PARAM_INPUT, null         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRORCAEVENTO'  , Param::PARAM_INPUT, null         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQORCAEVE'  , Param::PARAM_INPUT, null         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDDEPORCAEVE'  , Param::PARAM_INPUT, null         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQORCAMENTO', Param::PARAM_INPUT, null         , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMENSERRO'    , Param::PARAM_OUTPUT, ''          , Param::PARAM_TYPE_STR, 1000));


            $retorno = $procedure->execute(); 
            $this->entityManager->getConnection()->commit();
            return $retorno;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            //return $e->getMessage();
            throw $e;
        }
    }
    
    public function geraReserva($cdfilial, $nrpedido){
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'INSERE_RESERPRODPED');
            
            $procedure->addParam(new Param('P_CDFILIAL'   , Param::PARAM_INPUT, $cdfilial     , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPEDIDO'   , Param::PARAM_INPUT, $nrpedido     , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRITEMPED'  , Param::PARAM_INPUT, null                   , Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TODOS'      , Param::PARAM_INPUT, 'S'                    , Param::PARAM_TYPE_STR));

            $procedure->execute(); 

        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function selectFilial($cdfilial){
        try{
            $sql = "SELECT
                      CAST(NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) AS VARCHAR(4)) AS CDFILCENTCMP
                    FROM 
                      FILIAL FILCENT,
                      FILIAL 
                    WHERE
                      FILIAL.CDFILIAL = :P_CDFILIAL AND
                      NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) = FILCENT.CDFILIAL";
            
            $params = array(
                'P_CDFILIAL' => $cdfilial
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function buscaDataPedidoSoli($nrsolicmp, $cdfilsoli, $cdfillan){
        try{
            $sql = "SELECT DISTINCT(DTPROGENTR) FROM (SELECT * FROM ITEMSOLI
                    WHERE NRSOLICMP = :P_NRSOLICMP
                    AND CDFILSOLI = :P_CDFILSOLI
                    AND CDFILLAN = :P_CDFILLAN)
                    ";
            
            $params = array(
                'P_NRSOLICMP' => $nrsolicmp,
                'P_CDFILSOLI' => $cdfilsoli,
                'P_CDFILLAN' => $cdfillan
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function bustaItemSoliPorData($nrsolicmp, $cdfilsoli, $cdfillan, $dtprogentr){
        try{
            $sql = "SELECT * FROM ITEMSOLI
                    WHERE NRSOLICMP = :P_NRSOLICMP
                    AND CDFILSOLI = :P_CDFILSOLI
                    AND CDFILLAN = :P_CDFILLAN
                    AND DTPROGENTR = :P_DTPROGENTR";
            
            $params = array(
                'P_NRSOLICMP' => $nrsolicmp,
                'P_CDFILSOLI' => $cdfilsoli,
                'P_CDFILLAN' => $cdfillan,
                'P_DTPROGENTR' => $dtprogentr
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }


};